///////////////////////////////////////////////////////////////////////////
// File: BISC_Reader.h
// Project: BIS C handheld
// 
// Producer:	Balluff GmbH
// Author:		Roland Borbely, Burkhard Breuer
// Date:		21/02/2019
// 
//////////////////////////////////////////////////////////////////////////



#ifndef _BISC_Reader_
#define _BISC_Reader_

#pragma once


////////////////////////////////////////////////////////////////
// ERROR Codes
//
// According to BALLUFF 007 protocol. 
////////////////////////////////////////////////////////////////


#define ERR_NOERROR					0				
#define ERR_NO_DC_PRESENT			1				
#define ERR_READ					2
#define ERR_READ_DC_REMOVED			3
#define ERR_WRITE					4
#define ERR_WRITE_DC_REMOVED		5
#define ERR_INTERFACE				6
#define ERR_TELEGRAM_FORMAT			7
#define ERR_BCC						8
#define ERR_CABLE					9
#define ERR_NEW_CMD_READ			10
#define ERR_NEW_CMD_WRITE			11
#define ERR_NEW_CMD_HEADSELECT		12
#define ERR_HEAD_COMMUNICATION		13
#define ERR_DC_CRC					14
#define ERR_DC_ADDR					15
#define ERR_DC_FUNC_NOT_SUPPORTED	16
#define ERR_EEPROM					18
#define ERR_LOCK					19


//////////////////////////////////////////////////////////
// Internal DLL errors
// These error codes are used only within this DLL
//////////////////////////////////////////////////////////

#define ERR_UNKNOWN							-1		//unknown error

#define ERR_COMPORT_USED					49		//COM port is busy
#define ERR_RWHEAD_USED						50		//RW head is busy
#define ERR_COMPORT_OPENED					51		//COM port is opened
#define ERR_COMPORT_CLOSED					52		//COM port is not opened
#define ERR_COMPORT_EXIST					53		//COM port exists
#define ERR_COMPORT_NOT_EXIST				54		//COM port doesn't exist
#define ERR_READER_EXIST					55		//The R/W head is found -> exists
#define ERR_READER_NOT_EXIST				56		//The R/W head is NOT found -> doesn't exist
#define ERR_READER_OPENED					57		//R/W head connection opened successfully
#define ERR_READER_OPENING					58		//R/W head connection couldn't be opened
#define ERR_READER_CLOSED					59		//R/W head connection opened successfully
#define ERR_READER_CLOSING					60		//R/W head connection couldn't be opened
#define ERR_DLL_VERSION_EXIST				61		//DLL version information exists
#define ERR_OTHER_PORTS_USED				62		//This process uses other COM ports
#define ERR_FREE_PORT_OBJECT				63		//At least one free Serial port object exists
#define ERR_NO_FREE_PORT_OBJECT				64		//At least one free Serial port object exists
#define ERR_NO_ANSWER						65		//No answer from the BIS R/W head
#define ERR_INVALID_COMM_HANDLE				66		//The communication handle passed is invalid
#define ERR_READER_INIT_FAILED				67		//Reader initialization process failed
#define ERR_DATACARRIER_INIT_FAILED			68		//Data carrier initialization process failed
#define ERR_DATA_OVER_1023_BYTES			69		//Datalength is over 1024 bytes
#define ERR_TOO_SMALL_DATA					70		//Datalength is 0
#define ERR_TEST_READER						71		//Reader test is failed
#define ERR_UNKNOWN_CODETAG					72		//Unknown Code Tag type
#define ERR_TURN_ON_ANT						73		//Turn On the antenna failed
#define ERR_TURN_OFF_ANT					74		//Turn Off the antenna failed
#define ERR_CHANGE_READER_KEY				75		//Error changing READER key
#define ERR_CHANGE_TAG_KEY					76		//Error changing TAG key
#define ERR_WRONG_KEY_LENGTH				77		//Too short or too long key
#define ERR_WRONG_CODETAG_TYPE				78		//The codetag doesn't maintain this function
#define ERR_UNLOAD_USB						79		//USB unload is failed
#define ERR_RELOAD_USB						80		//USB reload is failed
#define ERR_USBPORT_INIT					81		//USB port initialization failed
#define ERR_POWEROFF_RWHEAD					82		//Power Off RW head failed
#define ERR_READ_COMPORT					83		//Read COM port failed
#define ERR_WRITE_COMPORT					84		//Write COM port failed
#define ERR_INVALID_PARAMETER				85		//Invalid parameter found
#define ERR_MEMORY_LOCK						86		//Error during Code Tag memory optimization


//////////////////////////////////////////////////////
// Predefined values
//////////////////////////////////////////////////////

typedef enum
{
	BIS_C_Unknown		= 0x00,
	BIS_C_1xx_04_05		= 0x01,
	BIS_C_1xx_11_32,

} CodeTags;

//CRC16
#define CRC16_NO		false		//read/write without CRC16
#define CRC16_YES		true		//read/write with CRC16


//LOCKing
typedef enum 
{
	LOCK_NO				= 0x00,		//COM port is not locked
	LOCK_FOR_READ		= 0x01,		//COM port is locked for read
	LOCK_FOR_WRITE		= 0x02,		//COM port is locked for write
	LOCK_UNKNOWN		= 0x04		//Unknown if the COM port is locked or not

}Locking;

typedef enum
{
	_4_Bytes			= 0x01,
	_8_Bytes,
	_16_Bytes

} LockArea;


///////////////////////////////////////////////////////
// BIS Structures
///////////////////////////////////////////////////////
typedef struct BIS_DLL_Version
{
	TCHAR		Version[12];				//4 digits for versioning
	TCHAR		DateOfRelease[11];		//Date of release of the BIS DLL
	TCHAR		SystemType[2];			//System L, M or C

} BISVERSION, *lpBISVERSION;

typedef struct ConnectionHandle
{
	UINT		COMPort;			//Number of the used COM port
	UINT		LOCK;				//Indicates whether a command runs on the given port
	DWORD		Process;			//Process handle
	CodeTags	CodeTagType;		//Type of the Codetag
	bool		CRCCheck;			//Use CRC check

} CONNHND, *lpCONNHND;


extern "C"
{
	DLLEXPORT	int BIS_USBPortState(unsigned int PortNumber);
	DLLEXPORT	int BIS_OpenUSBPort(unsigned int PortNumber, CONNHND& ConnHnd);
	DLLEXPORT	int BIS_CloseUSBPort(CONNHND& ConnHnd);
	DLLEXPORT	int BIS_OpenReader(CONNHND& ConnHnd, bool bCRC16, CodeTags DataCarrierType);
	DLLEXPORT	int BIS_CloseReader(CONNHND& ConnHnd);
	DLLEXPORT	int BIS_InitReader(CONNHND& ConnHnd, bool bCRC16, CodeTags DataCarrierType);
	DLLEXPORT   int BIS_ResetReader(CONNHND ConnHnd);
	DLLEXPORT	int BIS_ReadData(CONNHND ConnHnd, unsigned int Address, unsigned int Datalength, TCHAR* DataBuffer);
	DLLEXPORT	int BIS_WriteData(CONNHND ConnHnd, unsigned int Address, TCHAR* DataBuffer, unsigned int Datalength);
	DLLEXPORT	int BIS_WritePattern(CONNHND ConnHnd, unsigned int Address, unsigned int Patternlength, TCHAR Pattern);
	DLLEXPORT	int BIS_InitDataCarrier(CONNHND ConnHnd, unsigned short DataLength);
	DLLEXPORT	int BIS_MemoryLock(CONNHND ConnHnd, bool bLock, LockArea LockSize);
	DLLEXPORT	int BIS_UnLoadUSB(CONNHND ConnHnd);
	DLLEXPORT	int BIS_ReLoadUSB(CONNHND ConnHnd);
	DLLEXPORT	int BIS_PowerOffRWHead();
	DLLEXPORT	int BIS_GetDLLVersion(BISVERSION& Version);
	DLLEXPORT	int BIS_GetLastError(void);
	DLLEXPORT	TCHAR* BIS_GetLastErrorText(void);		
  LLEXPORT int BIS_StopCommand(CONNHND ConnHnd, bool state);		

}

#endif	// _BISC_Reader_